// QConnection.h: interface for the QConnection class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_QCONNECTION_H__31AFB0A4_A6FD_11D2_89E9_980EE8619202__INCLUDED_)
#define AFX_QCONNECTION_H__31AFB0A4_A6FD_11D2_89E9_980EE8619202__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "QError.h"
#include "ldap.h"

#define UNDEF_CONN_ID -1
class QConnection
{
public:
	QConnection( long id, CString &host, int port, CString &user, CString &password );
	virtual ~QConnection();

public:
  // Async bind interface
  BOOL BindAsync( QError *&err, int nLdapVersion = LDAP_VERSION3 );
	BOOL RebindAsync( QError *&err, int nLdapVersion = LDAP_VERSION3 );
  BOOL BindWait( QError *&err );

  // Synchronous bind interface
  BOOL Bind( QError *&err );

public:
  // Connection status
  BOOL Bound();

	// Socket associated with the server connection (if bound)
	SOCKET Socket();

public:
  // Unbind
  void Unbind();

	// Select for reads on this connection
  int SelectRead( struct timeval *timeout );

	// Select for exceptions 
	int SelectExceptions( struct timeval *timeout );

public:
  // Compare connection specification
  BOOL Compare
    ( const CString &host, int port, const CString &user, const CString &password );

public:
  // Get the LDAP context structure
  LDAP *GetLDAPContext() { return ldapctx; } ;

	// Get the connection ID
	long ID() { return conn_id ; } ;

	// Get the connection details
	void GetConnection( CString &host, int &port, CString &user, CString &password )
		{
			host = this->host;
			port = this->port;
			user = this->host;
			password = this->password;
		};

public:
	// Reference counting
	void AddRef() { ref_count++; } ;
	long LoseRef()
		{
			if ( ref_count > 0 )
				ref_count--;

			return ref_count ;
		} ;

public:
	// Last active timestamp stuff used to manage idle connections
	time_t LastActive() { return m_nLastActive; } ;
	void UpdateLastActive() { m_nLastActive = time( NULL ); } ;

public:
	// Dump the connection to log file
	void Dump();

private:
	// Socket close notifier called by QSocket
	void OnClose( int nErrorCode );

private:
  // Connection ID
  long conn_id;

  // Connection state
  BOOL bound;
  BOOL binding;

  // Start of idle time (unused at present)
  time_t m_nLastActive;
  
  // Count of connection references
  int ref_count;

  // LDAP message ID used during async bind
  int bind_msgid;

  // Host details
  CString host;
  int port;

  // Authentication details
  CString user;
  CString password;

  // LDAP context structure
  LDAP *ldapctx;
  LDAP *oldldapctx;	// Used when re-binding
};

#endif // !defined(AFX_QCONNECTION_H__31AFB0A4_A6FD_11D2_89E9_980EE8619202__INCLUDED_)
