///////////////////////////////////////////////////////////
//
// ptrlist.h
//
// Template for lists of pointers to any class.
//

#ifndef __MPtrList__
#define __MPtrList__

//
// Simple wrapper for a list of char *'s (treated as void *'s here)
class StrList : public CPtrList
{
public:
  StrList() : CPtrList(16)
    {
      ;
    }

  ~StrList()
    {
      if (!IsEmpty())
        {
          POSITION pos = GetHeadPosition();
          while (pos != NULL)
            {
              void *ptr = GetNext(pos);
              if (ptr != NULL)
                free(ptr);
            }
        }
    }
};

//
// Template for generalised list of of typed class pointers
template <class T> class AFX_EXT_CLASS PtrList
{
private:
  CPtrList *_list;

public: // Constructor and destructor

  PtrList(void);
  PtrList(int blocksize);
  ~PtrList();

public: // Public interface

  int GetCount();
  BOOL IsEmpty();
  T * GetHead();
  T * GetTail();
  T * RemoveHead();
  T * RemoveTail();
  POSITION AddHead(T * newElement);
  POSITION AddTail(T * newElement);
  void RemoveAll();
  POSITION GetHeadPosition();
  POSITION GetTailPosition();
  T * GetNext(POSITION& rPosition);
  T * GetPrev(POSITION& rPosition);
  T * GetAt(POSITION position);
  void SetAt(POSITION pos, T * newElement);
  void RemoveAt(POSITION position);
  POSITION InsertBefore(POSITION position, T * newElement);
  POSITION InsertAfter(POSITION position, T * newElement);
  POSITION Find(T * searchValue, POSITION startAfter = NULL);
  POSITION FindIndex(int nIndex);
};

template <class T> PtrList<T>::PtrList(void)
{
  _list = new CPtrList(16);
}

template <class T> PtrList<T>::PtrList(int blocksize)
{
  _list = new CPtrList(blocksize);
}

template <class T> PtrList<T>::~PtrList()
{
  delete _list;
}

template <class T> int PtrList<T>::GetCount(void)
{
  return _list->GetCount();
}

template <class T> BOOL PtrList<T>::IsEmpty(void)
{
  return _list->IsEmpty();
}

template <class T> T * PtrList<T>::GetHead()
{
  return (T *) _list->GetHead();
}

template <class T> T * PtrList<T>::GetTail(void)
{
  return (T *) _list->GetTail();
}

template <class T> T * PtrList<T>::RemoveHead(void)
{
  return (T *) _list->RemoveHead();
}

template <class T> T * PtrList<T>::RemoveTail(void)
{
  return (T *) _list->RemoveTail();
}

template <class T> POSITION PtrList<T>::AddHead(T * newElement)
{
  return _list->AddHead((void *) newElement);
}

template <class T> POSITION PtrList<T>::AddTail(T * newElement)
{
  return _list->AddTail((void *) newElement);
}

template <class T> void PtrList<T>::RemoveAll(void)
{
  _list->RemoveAll();
}

template <class T> POSITION PtrList<T>::GetHeadPosition(void)
{
  return _list->GetHeadPosition();
}

template <class T> POSITION PtrList<T>::GetTailPosition(void)
{
  return _list->GetTailPosition();
}

template <class T> T * PtrList<T>::GetNext(POSITION& rPosition)
{
  return (T *) _list->GetNext(rPosition);
}

template <class T> T * PtrList<T>::GetPrev(POSITION& rPosition)
{
  return (T *) _list->GetPrev(rPosition);
}

template <class T> T * PtrList<T>::GetAt(POSITION position)
{
  return (T *) _list->GetAt(position);
}

template <class T> void PtrList<T>::SetAt(POSITION pos, T * newElement)
{
  _list->SetAt(pos, (void *) newElement);
}

template <class T> void PtrList<T>::RemoveAt(POSITION position)
{
  _list->RemoveAt(position);
}

template <class T> POSITION PtrList<T>::InsertBefore(POSITION position, T * newElement)
{
  return _list->InsertBefore(position, (void *) newElement);
}

template <class T> POSITION PtrList<T>::InsertAfter(POSITION position, T * newElement)
{
  return _list->InsertAfter(position, (void *) newElement);
}

template <class T> POSITION PtrList<T>::Find(T * searchValue, POSITION startAfter)
{
  return _list->Find((void *) searchValue, startAfter);
}

template <class T> POSITION PtrList<T>::FindIndex(int nIndex)
{
  return _list->FindIndex(nIndex);
}

#endif // __MPtrList__
