// QError.cpp: implementation of the QError class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Query32.h"
#include "QError.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

QError::QError( int nErrCode )
{
  m_nErrCode = nErrCode;
	m_nOpCode = -1;
	time( &m_timestamp );
}

void QError::SetLdapError( int ldaperr, LDAPMessage *lm )
{
	time( &m_timestamp );
	m_nOpCode = -1;

  if ( ldaperr == LDAP_SUCCESS )
    m_nErrCode = QERR_OK;

  switch ( ldaperr )
    {
    case LDAP_OPERATIONS_ERROR:
      m_nErrCode = QERR_SERVICE;
      break; // LDAP_OPERATIONS_ERROR
      
    case LDAP_PROTOCOL_ERROR:
      m_nErrCode = QERR_NETWORK_ERROR;
      break; // LDAP_PROTOCOL_ERROR
      
    case LDAP_TIMELIMIT_EXCEEDED:
      m_nErrCode = QERR_TIME;
      break; // LDAP_TIMELIMIT_EXCEEDED
      
    case LDAP_SIZELIMIT_EXCEEDED:
      m_nErrCode = QERR_SIZE;
      break; // LDAP_SIZELIMIT_EXCEEDED

    case LDAP_COMPARE_FALSE:
      m_nErrCode = QERR_NOTHING_FOUND;
      break; // LDAP_COMPARE_FALSE

    case LDAP_STRONG_AUTH_NOT_SUPPORTED:
      m_nErrCode = QERR_SECURITY;
      break; // LDAP_STRONG_AUTH_NOT_SUPPORTED

    case LDAP_STRONG_AUTH_REQUIRED:
      m_nErrCode = QERR_SECURITY;
      break; // LDAP_STRONG_AUTH_REQUIRED

    case LDAP_NO_SUCH_ATTRIBUTE:
      m_nErrCode = QERR_ATTRIBUTE;
      break; // LDAP_NO_SUCH_ATTRIBUTE

    case LDAP_UNDEFINED_TYPE:
      m_nErrCode = QERR_ATTRIBUTE;
      break; // LDAP_UNDEFINED_TYPE

    case LDAP_INAPPROPRIATE_MATCHING:
      m_nErrCode = QERR_NOTHING_FOUND;
      break; // LDAP_INAPPROPRIATE_MATCHING

    case LDAP_CONSTRAINT_VIOLATION:
      m_nErrCode = QERR_SERVICE;
      break; // LDAP_CONSTRAINT_VIOLATION

    case LDAP_TYPE_OR_VALUE_EXISTS:
      m_nErrCode = QERR_UPDATE;
      break; // LDAP_TYPE_OR_VALUE_EXISTS

    case LDAP_INVALID_SYNTAX:
      m_nErrCode = QERR_ATTRIBUTE;
      break; // LDAP_INVALID_SYNTAX
 
    case LDAP_NO_SUCH_OBJECT:
      m_nErrCode = QERR_NAME;
      break; // LDAP_NO_SUCH_OBJECT

    case LDAP_ALIAS_PROBLEM:
      m_nErrCode = QERR_NAME;
      break; // LDAP_ALIAS_PROBLEM

    case LDAP_INVALID_DN_SYNTAX:
      m_nErrCode = QERR_NAME;
      break; // LDAP_INVALID_DN_SYNTAX

    case LDAP_IS_LEAF:
      m_nErrCode = QERR_SERVICE;
      break; // LDAP_IS_LEAF

    case LDAP_ALIAS_DEREF_PROBLEM:
      m_nErrCode = QERR_NAME;
      break; // LDAP_ALIAS_DEREF_PROBLEM

    case LDAP_INAPPROPRIATE_AUTH:
      m_nErrCode = QERR_SECURITY;
      break; // LDAP_INAPPROPRIATE_AUTH

    case LDAP_INVALID_CREDENTIALS:
      m_nErrCode = QERR_SECURITY;
      break; // LDAP_INVALID_CREDENTIALS

    case LDAP_INSUFFICIENT_ACCESS:
      m_nErrCode = QERR_SECURITY;
      break; // LDAP_INSUFFICIENT_ACCESS

    case LDAP_BUSY:
      m_nErrCode = QERR_SERVICE;
      break; // LDAP_BUSY

    case LDAP_UNAVAILABLE:
      m_nErrCode = QERR_SERVICE;
      break; // LDAP_UNAVAILABLE

    case LDAP_UNWILLING_TO_PERFORM:
      m_nErrCode = QERR_SERVICE;
      break; // LDAP_UNWILLING_TO_PERFORM

    case LDAP_LOOP_DETECT:
      m_nErrCode = QERR_SERVICE;
      break; // LDAP_LOOP_DETECT

    case LDAP_NAMING_VIOLATION:
      m_nErrCode = QERR_SERVICE;
      break; // LDAP_NAMING_VIOLATION

    case LDAP_OBJECT_CLASS_VIOLATION:
      m_nErrCode = QERR_ATTRIBUTE;
      break; // LDAP_OBJECT_CLASS_VIOLATION

    case LDAP_NOT_ALLOWED_ON_NONLEAF:
      m_nErrCode = QERR_SERVICE;
      break; // LDAP_NOT_ALLOWED_ON_NONLEAF

    case LDAP_NOT_ALLOWED_ON_RDN:
      m_nErrCode = QERR_ATTRIBUTE;
      break; // LDAP_NOT_ALLOWED_ON_RDN

    case LDAP_ALREADY_EXISTS:
      m_nErrCode = QERR_ATTRIBUTE;
      break; // LDAP_ALREADY_EXISTS

    case LDAP_NO_OBJECT_CLASS_MODS:
      m_nErrCode = QERR_UPDATE;
      break; // LDAP_NO_OBJECT_CLASS_MOD

    case LDAP_OTHER:
      m_nErrCode = QERR_INTERNAL;
      break; // LDAP_OTHER

    case LDAP_SERVER_DOWN:
      m_nErrCode = QERR_LOST_SERVER_CONNECTION;
      break; // LDAP_SERVER_DOWN

    case LDAP_LOCAL_ERROR:
      m_nErrCode = QERR_INTERNAL;
      break; // LDAP_LOCAL_ERROR

    case LDAP_ENCODING_ERROR:
      m_nErrCode = QERR_INTERNAL;
      break; // LDAP_ENCODING_ERROR

    case LDAP_DECODING_ERROR:
      m_nErrCode = QERR_INTERNAL;
      break; // LDAP_DECODING_ERROR

    case LDAP_AUTH_UNKNOWN:
      m_nErrCode = QERR_SECURITY;
      break; // LDAP_AUTH_UNKNOWN

    case LDAP_FILTER_ERROR:
      m_nErrCode = QERR_FILTER_ERROR;
      break; // LDAP_FILTER_ERROR

    default:
      m_nErrCode = QERR_SERVICE;
      break;
    }
}

QError::~QError()
{
}

//////////////////////////////////////////////////////////////////////
// 
// QError::Print
// 
// Print the error as a "user friendly" string
void QError::Print( CString &str )
{
  WIN32_GUFF();

  str = "Unrecognized error!";
  if ( m_nErrCode == QERR_OK )
    {
      str = "Operation successful";
      return;
    }

  switch ( m_nErrCode )
    {      
    case QERR_ATTRIBUTE:
      str = "Invalid attribute supplied.";
      break;
      
    case QERR_NAME:
      str = "Invalid directory entry name supplied.";
      break;
      
    case QERR_SERVICE:
      str = "Service error, cannot perform.";
      break;
      
    case QERR_ABANDONED:
      str = "Request abandoned.";
      break;
      
    case QERR_SECURITY:
      str = "Security problem, you have insufficient rights or invalid credentials.";
      break;
      
    case QERR_ABANDONFAILED:
      str = "Couldn't abandon request.";
      break;
      
    case QERR_UPDATE:
      str = "Couldn't update entry.";
      break;
      
    case QERR_DSAREFERRAL:
      str = "Couldn't refer request.";
      break;

    case QERR_NETWORK_ERROR:
      str = "Network problem, cannot perform.";
      break;
      
    case QERR_NOT_BOUND:
      str = "Not connected to directory.";
      break;
      
    case QERR_INTERNAL_LIMIT:
      str = "Internal limit reached.";
      break;
      
    case QERR_INTERNAL:
      str = "Internal error detected.";
      break;
      
    case QERR_INVALID_ARGUMENT:
      str = "Invalid information supplied.";
      break;
      
    case QERR_INSUFFICIENT_DATA:
      str = "Not enough search data supplied.";
      break;
      
    case QERR_NOTHING_FOUND:
      str = "Nothing found with given values.";
      break;
      
    case QERR_TIME:
      str = "Time limit reached.";
      break;
      
    case QERR_SIZE:
      str = "Data retrieval limit reached.";
      break;
      
    case QERR_ADMIN:
      str = "Administrative limit reached.";
      break;

    case QERR_SVC_BUSY:
      str = "The directory service is busy at present.";
      break;

    case QERR_SEARCHES_FAILED:
      str = "Some search operations failed.";
      break;

    case QERR_LOST_SERVER_CONNECTION:
      str = 
				"A directory server connection was lost.\
				 this may be due to a server or network error.";

			break;

    case QERR_BIND_FAILED:
      str = 
				"Couldn't connect to the directory server.\
				 This may be due to a server or network error.";

      break;

    case QERR_API_ERROR:
      str = "An application interface error occurred!";
			break;
    }
}

//////////////////////////////////////////////////////////////////////
// 
// QError::GetTimestamp
void QError::GetTimestamp( time_t &rTS )
{
	rTS = m_timestamp;
}

//////////////////////////////////////////////////////////////////////
// 
// QError::SetOpInfo
void QError::SetOpInfo( int nOpCode, QDN &rDN, CString &rHostName, int nPortNum )
{
	m_nOpCode = nOpCode;
	m_DN = rDN;
	m_hostName = rHostName;
	m_portNum = nPortNum;
}

//////////////////////////////////////////////////////////////////////
// 
// QError::GetOpInfo
BOOL QError::GetOpInfo( int &rOpCode, QDN &rDN, CString &rHostName, int &rPortNum )
{
	if ( m_nOpCode == -1 )
		return FALSE;

	rOpCode = m_nOpCode;
	rDN = m_DN;
	rHostName = m_hostName;
	rPortNum = m_portNum;

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// 
// QError::Copy
QError *QError::Copy()
{
	QError *pErr = new QError( ErrorCode() );

	if ( m_nOpCode >= 0 )
		pErr->SetOpInfo( m_nOpCode, m_DN, m_hostName, m_portNum );

	return pErr;
}

