// QRuleSet.cpp: implementation of the QRuleSet class. QRuleSet
// groups a set of search classes to form a set of search heurisitcs.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Query32.h"
#include "QRuleSet.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
QRuleSet::QRuleSet( const CString &name, PtrList<QSearchClass> &class_list_in )
{
  this->name = name;

  POSITION pos = class_list_in.GetHeadPosition();
  while ( pos != NULL )
    this->srch_class_list.AddTail( class_list_in.GetNext( pos ) );
}

QRuleSet::~QRuleSet()
{
  POSITION pos = srch_class_list.GetHeadPosition();
  while ( pos != NULL )
    {
      delete srch_class_list.GetNext( pos );
    }
}

//////////////////////////////////////////////////////////////////////
// Return the list of search classes.
void QRuleSet::GetClassList( PtrList<QSearchClass> &class_list_out )
{
  POSITION pos = srch_class_list.GetHeadPosition();
  while ( pos != NULL )
    class_list_out.AddTail( srch_class_list.GetNext( pos ) );
}

//////////////////////////////////////////////////////////////////////
// Return the named search class.
QSearchClass *QRuleSet::GetNamedClass( CString &name )
{
  POSITION pos = srch_class_list.GetHeadPosition();
  while ( pos != NULL )
    {
      QSearchClass *sc = srch_class_list.GetNext( pos );
      if ( sc->Name() == name )
        return sc;
    }

  return FALSE;
}

//////////////////////////////////////////////////////////////////////
// Return the search class that matches the given entry's object class.
QSearchClass *QRuleSet::FindMatchingClass( QEntry *entry )
{
  QAttr *oc_attr = entry->ObjectClass();
  POSITION pos = srch_class_list.GetHeadPosition();
  while ( pos != NULL )
    {
      QSearchClass *sc = srch_class_list.GetNext( pos );
      if ( sc->HasObjectClass( oc_attr ) == TRUE )
        return sc;
    }

  return NULL;
}
